function y=OPTAfading(N,SigmaX,SigmaN);

% Function to calculate the OPTA curves with fading in the channel,
% assuming the fading change for each retransmission
% Example: y=OPTAfading2(20000,0,0);

EsN0=linspace(0,20,50);
M=[4,16];

SNRri=zeros(length(M),length(EsN0));
SNRra=zeros(length(M),length(EsN0));

% Rician fading with low variance
[fading, tc]=genfading('rice',10,1/44100,N*max(M),0.01,100);
%fading=fading/mean(fading);fading=fading/2+(1-1/2);
%var(fading),10*log10(1/var(fading))

% Rayleigh fading
[fadingr, tcr]=genfading('rayleigh',10,1/44100,N*max(M),0.01);
%10*log10(var(fading)),mean(fading)


for j=1:length(M)
    for i=1:length(EsN0)
        SNRri(j,i)=fadingfeedback(EsN0(i),M(j),N,SigmaX,SigmaN,fading);    
    end
end
for j=1:length(M)
    for i=1:length(EsN0)
        SNRra(j,i)=fadingfeedback(EsN0(i),M(j),N,SigmaX,SigmaN,fadingr);    
    end
end
%------------------------OPTA with ideal feddback-------------------------
EsN0l=10.^(EsN0/10);
SNRo=zeros(length(M),length(EsN0));
for i=1:length(M)
    SNRo(i,:)=(1+2*EsN0l/M(i)).^M(i);
end
SNRo=10*log10(SNRo);
%--------------------------------------------------------------------------

figure();
subplot(2,2,1);
plot(10*log10(fading));
axis([0 N*4 -20 5]); %For rician fading
xlabel('samples'),ylabel('dB')

subplot(2,2,2);hold on;
plot(EsN0,SNRri(1,:));
plot(EsN0,SNRri(2,:),'r');
plot(EsN0,SNRo(1,:),'k--');
plot(EsN0,SNRo(2,:),'k--');
grid on;axis([0 20 0 80]);
xlabel('E_s/N_0 (dB)'),ylabel('SNR(dB)');
hold off;

subplot(2,2,3);
plot(10*log10(fadingr));
axis([0 N*4 -20 5]); %For rayliegh fading
xlabel('samples'),ylabel('dB')

subplot(2,2,4);hold on;
plot(EsN0,SNRra(1,:));
plot(EsN0,SNRra(2,:),'r');
plot(EsN0,SNRo(1,:),'k--');
plot(EsN0,SNRo(2,:),'k--');
grid on;axis([0 20 0 80]);
xlabel('E_s/N_0 (dB)'),ylabel('SNR(dB)');
hold off;

y=1;



function SNR=fadingfeedback(Es,M,N,SigmX,SigmN,fading)

% Es(dB): Energy per source sample (0~20 dB)
% M: Expansion factor
% N: Number of samles
% SigmaX: Signal variance
% SigmaN: Channel noise power

Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);

%--------------------Calculate coeficients----------------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=1;B(1)=1;D(1)=SigmaX;
for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------

x=randn(1,N)*sqrt(SigmaX);
noise=randn(M+1,N)*sqrt(SigmaN);
out=zeros(1,N);u=0;

% fading=genRayleighSamples(1024,ceil(10000/1024),1e4,100);
% fading=fading*fadingpow;
% varian=10*log10(var(fading));

% fading=randn(1,N)*fadingpow;
% varian=10*log10(var(fading));


for n=1:N
    d=0;
    y=0;
    for i=2:M+1
        u=u+1;
        a=(x(n)-d)*A(i);
        b=a*fading(u)+noise(i,n);
        c=b*B(i);
        y=c+y;
        d=y*fading(u);
    end
    out(n)=y;
end

outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));