function y=OPTAfadingbeta(N,SigmaX,SigmaN,delta);

% This function returns the OPTA curves for M=4 and M=16 and thr curves
% obtained from introducing fading in the channel and then corrected
% Example: y=OPTAnoise(20000,0,0);

figure();

EsN0=linspace(0,20,50);
M=[4,16];
SNRf=zeros(length(M),length(EsN0));
SNRf2=zeros(length(M),length(EsN0));
power1=zeros(length(M),length(EsN0));
power2=zeros(length(M),length(EsN0));

% Rician fading with low variance
[fading, tc]=genfading('rice',5,1/44100,N*2*max(M),delta,100);
fading=fading/mean(fading);
fading=fading/2+(1-1/2);
%var(fading),10*log10(1/var(fading)),tc

subplot(2,2,1);plot(10*log10(fading));axis([0 N*4 -20 5]);
xlabel('samples'),ylabel('dB')

% Rayleigh fading
[fading2, tc2]=genfading('rayleigh',5,1/44100,N*2*max(M),delta);
%10*log10(var(fading2)),mean(fading2),tc2

subplot(2,2,3);plot(10*log10(fading2));axis([0 N*4 -20 5]);
xlabel('samples'),ylabel('dB')


%----------------------------Curves calculation----------------------------
tc=[16*4*2,16*2];
for j=1:length(M)
    for i=1:length(EsN0)
        [SNRf(j,i),power1(j,i)]=fadingfeedback(EsN0(i),M(j),N,SigmaX,SigmaN,fading,tc(j));    
    end
end
tc=[2*4*2,2*2];
for j=1:length(M)
    for i=1:length(EsN0)
        [SNRf2(j,i),power2(j,i)]=fadingfeedback(EsN0(i),M(j),N,SigmaX,SigmaN,fading2,tc(j));    
    end
end
EsN0(26)
10*log10(power2(:,26))

%------------------------OPTA with ideal feddback-------------------------
EsN0l=10.^(EsN0/10);
SNRo=zeros(length(M),length(EsN0));
for i=1:length(M)
    SNRo(i,:)=(1+2*EsN0l/M(i)).^M(i);
end
SNRo=10*log10(SNRo);
%--------------------------------------------------------------------------

subplot(2,2,2);hold on;

plot(EsN0,SNRf(1,:));
plot(EsN0,SNRf(2,:),'r');

plot(EsN0,SNRo(1,:),'k--');
plot(EsN0,SNRo(2,:),'k--');

grid on;axis([0 20 0 80]);
xlabel('E_s/N_0 (dB)'),ylabel('SNR(dB)');
hold off;


subplot(2,2,4);hold on;

plot(EsN0,SNRf2(1,:));
plot(EsN0,SNRf2(2,:),'r');

plot(EsN0,SNRo(1,:),'k--');
plot(EsN0,SNRo(2,:),'k--');

grid on;axis([0 20 0 80]);
xlabel('E_s/N_0 (dB)'),ylabel('SNR(dB)');
hold off;


figure();
subplot(1,2,1);
plot(EsN0,10*log10(power1'))

subplot(1,2,2);
plot(EsN0,10*log10(power2'))
y=1;



function [SNR,power]=fadingfeedback(Es,M,N,SigmX,SigmN,fading,Tc)

% Es(dB): Energy per source sample (0~20 dB)
% M: Expansion factor
% N: Number of samles
% SigmaX: Signal variance
% SigmaN: Channel noise power

Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);

%--------------------Calculate coeficients----------------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=1;B(1)=1;D(1)=SigmaX;
for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------

x=randn(1,N)*sqrt(SigmaX);
noise=randn(M+1,N)*sqrt(SigmaN);
out=zeros(1,N);
power=zeros(1,N);
fadingcount=1;
pulse=3;
u=1;%to increase the fading

d=pulse*fading(1); %Send pulse to measure fading
betaa=pulse/d;
for n=1:N
    d=0;
    y=0;
    if fadingcount==Tc
        fadingcount=0;u=u+1;
        d=pulse*fading(u); %Send pulse to measure fading
        betaa=pulse/d;
    end
    
    pow=zeros(1,M);
    for i=2:M+1
        u=u+1;
        a=(x(n)-d)*A(i)*betaa;
        pow(i-1)=a;
        b=a*fading(u)+noise(i,n);
        c=b*B(i);
        y=c+y;
        d=y*fading(u)*betaa;
    end
    power(n)=sum(pow.^2);
    out(n)=y;
    fadingcount=fadingcount+1;
end

power=(1/(N*M))*sum(power);
outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));