function y=OPTAnoise(N,SigmaX,SigmaN);

% FUnction to calculate the OPTA curves with noise in the feedback channel
% Example: y=OPTAnoise(20000,0,0);

EsN0=linspace(0,20,50);

M=[4,16];
R=[5,10,15];
SNR=zeros(length(M),length(EsN0),length(R));
for j=1:length(M)
    for z=1:length(R)
        for i=1:length(EsN0)
            SNR(j,i,z)=realfeedback(EsN0(i),M(j),N,SigmaX,SigmaN,R(z));
        end
    end
end

%------------------------OPTA with ideal feddback-------------------------
EsN0l=10.^(EsN0/10);
SNRo=zeros(length(M),length(EsN0));
for i=1:length(M)
    SNRo(i,:)=(1+2*EsN0l/M(i)).^M(i);
end
SNRo=10*log10(SNRo);
%--------------------------------------------------------------------------

figure();
hold on;

plot(EsN0,SNR(1,:,1)');
plot(EsN0,SNR(1,:,2)','r');
plot(EsN0,SNR(1,:,3)','m');

plot(EsN0,SNR(2,:,1)','b-.');
plot(EsN0,SNR(2,:,2)','r-.');
plot(EsN0,SNR(2,:,3)','m-.');

plot(EsN0,SNRo(1,:)','k--');
plot(EsN0,SNRo(2,:)','k--');

grid on;axis([0 20 0 50]);
xlabel('E_s/N_0 (dB)'),ylabel('SNR(dB)');

hold off;
y=1;




function SNR=realfeedback(Es,M,N,SigmX,SigmN,R)

% Es(dB): Energy per source sample (0~20 dB)
% M: Expansion factor
% N: Number of samles
% SigmaX: Signal variance
% SigmaN: Channel noise power
% R(dB): Feedback channel power over feed forward channel

Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
Rl=10^(R/10);

A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=1;B(1)=1;D(1)=SigmaX;

for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

x=randn(1,N)*sqrt(SigmaX);
noise=randn(M+1,N)*sqrt(SigmaN);
noise2=randn(M+1,N)*sqrt(SigmaN);
out=zeros(1,N);

for n=1:N
    f=0;
    y=0;
    for i=2:M+1       
        a=(x(n)-f)*A(i);
        b=a+noise(i,n);
        c=b*B(i);
        y=c+y;
        d=Rl*A(2)*y;
        e=d+noise2(i,n);
        f=e/(Rl*A(2));
    end
    out(n)=y;
end

outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));