function POWERvsI()

% Draws the evolution of the sent power through the several retransmission.
% Is a way to obtain the optimum expansion factor

Es=10;SigmX=0;SigmN=0;maxbeta=2;M=16;

Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
maxbeta=10^(maxbeta/10);
noise=randn(M+1,1)*sqrt(SigmaN);
%---------------------------Calculate%coefficients--------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=0;B(1)=0;D(1)=SigmaX;

for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------

[fading, tc]=genfading('rayleigh',1,1/(44100),1e2,0.01,100);
fa=ones(1,1e2);
x=0.6128214*1e3;

pulse=3;
d=pulse*fading(1); %Send pulse to measure fading
betaa=pulse/d;

p1=sendd(x,M,1,fa,noise,A,B);
p2=sendd(x,M,betaa,fading,noise,A,B);

figure();hold on;
plot(10*log10(p1.^2));
plot(10*log10(p2.^2),'r--');
xlabel('Retransmission-i'),ylabel('Power (dB)'),
legend('Without fading','With fading')





function pow=sendd(x,M,betaa,fading,noise,A,B)

% This function will transmit one sample x using M retransmissions. This
% sample is affected by the channel fading and noise
%
% x: is the sample to be sent
% betaa: amplification factor calculated to compensate the fading
% fading: channel fading for this time instant
% noise: vector which contains gaussian noise (M+1) samples
% A and B: vectors with the coefficient for the retransmissions
u=1;
p=zeros(1,M);
d=0;y=0;
for i=2:M+1
    u=u+1;
    a=(x-d)*betaa*A(i);
    %b=(a+noise(i,n))*fading(u);
    p(i-1)=a;
    c=(a*fading(u)+noise(i))*B(i);
    y=c+y;
    d=y*fading(u)*betaa;
end

pow=p;