function SNRvsM(N)

% Draws the curves that relate the SNR with the expansion factor used for
% several coherence times

EsN0=10;SigmaX=0;SigmaN=0;
M=1:40;%tc=[1 2 4 10 20];
tc=1:40;
SNR=zeros(length(tc),length(M));
x=randn(1,N)*sqrt(1);
noise=randn(40+1,N)*sqrt(1);

% Rician fading with low variance
[fading, tcc]=genfading('rice',5,1/44100,N*2*max(M),0.01,100);
fading=fading/mean(fading);fading=fading/2+(1-1/2);

powerri=zeros(length(tc),length(M));
for j=1:length(tc)
    for i=1:length(M)
        [SNR(j,i) powerri(j,i)]=fadingfeedback(x,EsN0,M(i),N,SigmaX,SigmaN,noise,fading,tc(j));    
    end
end


figure();

subplot(2,2,1)
plot(M,[SNR(1,:)' SNR(2,:)' SNR(5,:)' SNR(10,:)' SNR(20,:)' SNR(40,:)'])
grid on; xlabel('Expansion factor M'),ylabel('SNR (dB)')
legend('N_T_c=1','N_T_c=2','N_T_c=5','N_T_c=10','N_T_c=20','N_T_c=40','Location','SouthEast')
axis([1 35 0 80])

subplot(2,2,2)
maximum=zeros(1,length(tc));
for i=1:length(tc)
    a=SNR(i,:);
    maximum(i)=M(find(a==max(a)));
end
plot(tc,maximum);
xlabel('N_T_c'),ylabel('Optimum M')
axis([0 40 0 20])



%**************************************************************************
%**************************************************************************
SNR=zeros(length(tc),length(M));

%Rayleigh fading---------------------------------------------------
[fading, tcc]=genfading('rayleigh',1,1/44100,N*16*5,0.01);

powerra=zeros(length(tc),length(M));
for j=1:length(tc)
    for i=1:length(M)
        [SNR(j,i),powerra(j,i)]=fadingfeedback(x,EsN0,M(i),N,SigmaX,SigmaN,noise,fading,tc(j));    
    end
end

subplot(2,2,3)
plot(M,[SNR(1,:)' SNR(2,:)' SNR(5,:)' SNR(10,:)' SNR(20,:)' SNR(40,:)'])
grid on;xlabel('Expansion factor M'),ylabel('SNR (dB)')
legend('N_T_c=1','N_T_c=2','N_T_c=5','N_T_c=10','N_T_c=20','N_T_c=40','Location','SouthEast')
axis([0 35 0 80])

subplot(2,2,4)
maximum=zeros(1,length(tc));
for i=1:length(tc)
    a=SNR(i,:);
    maximum(i)=M(find(a==max(a)));
end
plot(tc,maximum);
xlabel('N_T_c'),ylabel('Optimum M')
axis([0 40 0 20])

%**************************************************************************
%**************************************************************************

figure();
subplot(1,2,1);
plot(M,10*log10([powerri(1,:)' powerri(2,:)' powerri(5,:)' powerri(10,:)' powerri(20,:)']))

subplot(1,2,2);
plot(M,10*log10([powerra(1,:)' powerra(2,:)' powerra(5,:)' powerra(10,:)' powerra(20,:)']))



function [SNR,power]=fadingfeedback(x,Es,M,N,SigmX,SigmN,noise,fading,Tc)

% Es(dB): Energy per source sample (0~20 dB)
% M: Expansion factor
% N: Number of samles
% SigmaX: Signal variance
% SigmaN: Channel noise power

Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);

%--------------------Calculate coeficients----------------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=1;B(1)=1;D(1)=SigmaX;
for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------

out=zeros(1,N);
fadingcount=1;
power=zeros(1,N);
pulse=3;
u=0;%to increase the fading

d=pulse*fading(1); %Send pulse to measure fading
betaa=pulse/d;
for n=1:N
    d=0;
    y=0;
    if fadingcount==Tc
        fadingcount=0;u=u+1;
        d=pulse*fading(u); %Send pulse to measure fading
        betaa=pulse/d;
    end
    pow=zeros(1,M);
    for i=2:M+1
        u=u+1;
        a=(x(n)-d)*A(i)*betaa;
        pow(i-1)=a;
        b=a*fading(u)+noise(i,n);
        c=b*B(i);
        y=c+y;
        d=y*fading(u)*betaa;
    end
    out(n)=y;
    power(n)=sum(pow.^2);
    fadingcount=fadingcount+1;
end

power=(1/(N*M))*sum(power);
outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));