function [out,SNR,buffhist,smallfading,power]=systembufferM(x,Es,SigmX,SigmN,noise,tc,fading,maxbeta)


% This function implements the system without noise in the feedback and
% with fading in the channel.
%
% x:input signal
% Es: Energy per source sample (0~20 dB)
% M: Expansion factor
% SigmaX: Signal variance
% SigmaN: Channel noise power
% tc (samples): Coherence time
% faiding: is a vector with the faiding components. Notice that this vector
%          has to be big enough (length(faiding)>>lengt(x))
% k1length & k2length: Are the length of the first and second part of the
%                      frame respectively
% pulse: value of the pulse sent from the receiver to the transmitter to
%        measure the fading in the channel
% flag: value of the flags used to indicate when the data is incoming. Has
% to be high but limitted by the receiver
% maxbeta (dB): maximum value of the apmlification factor to compensate the
%          fading. Is related to the buffer size.
%
% Usage example:
% SigmaN=10^(0/10);
% noise=randn(16+1,length(x))*sqrt(SigmaN);
% [fading, tc]=rayleighfading(9,1/(44100*16),1e6);
% [out,SNR,buffhist,power]=systmwithvariablem4(x,10,0,0,noise,tc,fading,2);

global buffreceiver
buffreciever=0;

x=(x(:))';
Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
maxbeta=10^(maxbeta/10);
N=length(x);
pulse=4;
flag=3;


%---------------------------Calculate%coefficients--------------------------
M=2;
A2=zeros(1,2+1);B2=zeros(1,2+1);D2=zeros(1,2+1);
A2(1)=0;B2(1)=0;D2(1)=SigmaX;
for i=2:(2+1)
    A2(i)=sqrt((2*Esl/M)/D2(i-1));
    B2(i)=sqrt(D2(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D2(i)=D2(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=4;
A4=zeros(1,4+1);B4=zeros(1,4+1);D4=zeros(1,4+1);
A4(1)=0;B4(1)=0;D4(1)=SigmaX;
for i=2:(4+1)
    A4(i)=sqrt((2*Esl/M)/D4(i-1));
    B4(i)=sqrt(D4(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D4(i)=D4(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=6;
A6=zeros(1,6+1);B6=zeros(1,6+1);D6=zeros(1,6+1);
A6(1)=0;B6(1)=0;D6(1)=SigmaX;
for i=2:(6+1)
    A6(i)=sqrt((2*Esl/M)/D6(i-1));
    B6(i)=sqrt(D6(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D6(i)=D6(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=8;
A8=zeros(1,8+1);B8=zeros(1,8+1);D8=zeros(1,8+1);
A8(1)=0;B8(1)=0;D8(1)=SigmaX;
for i=2:(8+1)
    A8(i)=sqrt((2*Esl/M)/D8(i-1));
    B8(i)=sqrt(D8(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D8(i)=D8(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=10;
A10=zeros(1,10+1);B10=zeros(1,10+1);D10=zeros(1,10+1);
A10(1)=0;B10(1)=0;D10(1)=SigmaX;
for i=2:(10+1)
    A10(i)=sqrt((2*Esl/M)/D10(i-1));
    B10(i)=sqrt(D10(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D10(i)=D10(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=12;
A12=zeros(1,12+1);B12=zeros(1,12+1);D12=zeros(1,12+1);
A12(1)=0;B12(1)=0;D12(1)=SigmaX;
for i=2:(12+1)
    A12(i)=sqrt((2*Esl/M)/D12(i-1));
    B12(i)=sqrt(D12(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D12(i)=D12(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=14;
A14=zeros(1,14+1);B14=zeros(1,10+1);D14=zeros(1,10+1);
A14(1)=0;B14(1)=0;D14(1)=SigmaX;
for i=2:(14+1)
    A14(i)=sqrt((2*Esl/M)/D14(i-1));
    B14(i)=sqrt(D14(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D14(i)=D14(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=16;
A16=zeros(1,16+1);B16=zeros(1,16+1);D16=zeros(1,16+1);
A16(1)=0;B16(1)=0;D16(1)=SigmaX;
for i=2:(16+1)
    A16(i)=sqrt((2*Esl/M)/D16(i-1));
    B16(i)=sqrt(D16(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D16(i)=D16(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------
% Inizialize all variables

buffhist=[];power=zeros(1,N);buff=[];
out=zeros(1,N);smallfading=zeros(1,N);
i=0;u=0;% i and u point to the last used sample in x and fading respectively
Mcount=0;% to count total M
kt=tc; %Mmax=16
k2=tc*(16/2);k4=tc*(16/4);k6=floor(tc*16/6);k8=tc*(16/8);
k10=floor(tc*16/10);k12=floor(tc*16/12);k14=floor(tc*16/14);
buffreceiver=0;

while (N-i+length(buff))>=kt
    
    % Buffer k samples kt=(tc-1)/Mmax
    if i+kt<=N
        buff=[buff x(i+1:i+kt)];
        i=i+kt;
        buffreceiver=buffreceiver+kt;
    end
    if length(buff)<(kt)
        % If we are in this point is because (N-i+length(buff))>kt, but
        % there are les than kt-1 samples to be sent, so we have to buffer
        % untill kt
        aux=length(buff);
        buff=[buff x(i+1:i+kt-aux)];
        i=i+kt-aux;
        buffreceiver=kt;
    end
    
    %-------------------Calculate amplification factor---------------------
    u=u+1;
    d=pulse*fading(u); %Send pulse to measure fading
    betaa=pulse/d;
    while betaa>=maxbeta
        u=u+kt*16; %We advance tc, which means kt*Mmax samples
        d=pulse*fading(u);
        betaa=pulse/d;
        %If betaa>maxbeta is because exists outage and wehave to resend the
        %pulse each tc
        %No we have to buffer the corresponding samples to tc --> kt
        if i+kt<=N
            buff=[buff x(i+1:i+kt)];
            i=i+kt;
            buffreceiver=buffreceiver+kt;
        end
    end
    %----------------------------------------------------------------------
    
    %Now the reciver has to know that we are sending nformation again.
    
    %-------------------------------Send flag------------------------------
    u=u+1;
    ff=flag*betaa*fading(u)+rand(1)*sqrt(SigmaN);
    if ff<2
        disp('Error in flag reception. The system does not work')
        return;
    end
    %-------------------------------------------------------------------
    len=length(buff);
    % Transmit k samples
    if len<=k14
        %M=16--------------------------------------------------------------
        Mcount=Mcount+16*kt;
        pow=zeros(1,kt);
        for w=1:kt
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+16),noise(:,i-len+w),A16,B16,tc);
            u=u+16;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:kt)=[];
        buffreceiver=buffreceiver-kt;
        
    elseif len>k14 && len<=300
        %M=14--------------------------------------------------------------
        Mcount=Mcount+14*k14;
        pow=zeros(1,k14);
        for w=1:k14
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+14),noise(:,i-len+w),A14,B14,tc);
            u=u+14;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k14)=[];
        buffreceiver=buffreceiver-k14;
        
    elseif len>300 && len<=600
        %M=12--------------------------------------------------------------
        Mcount=Mcount+12*k12;
        pow=zeros(1,k12);
        for w=1:k12
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+12),noise(:,i-len+w),A12,B12,tc);
            u=u+12;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k12)=[];
        buffreceiver=buffreceiver-k12;
        
    elseif len>600 && len<=1000
        %M=10--------------------------------------------------------------
        Mcount=Mcount+10*k10;
        pow=zeros(1,k10);
        for w=1:k10
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+10),noise(:,i-len+w),A10,B10,tc);
            u=u+10;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k10)=[];
        buffreceiver=buffreceiver-k10;
        
    elseif len>1000&& len<=1500
        %M=8---------------------------------------------------------------
        Mcount=Mcount+8*k8;
        pow=zeros(1,k8);
        for w=1:k8
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+8),noise(:,i-len+w),A8,B8,tc);
            u=u+8;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k8)=[];
        buffreceiver=buffreceiver-k8;
        
    elseif len>1500 && len<=2100
        %M=6--------------------------------------------------------------
        Mcount=Mcount+6*k6;
        pow=zeros(1,k6);
        for w=1:k6
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+6),noise(:,i-len+w),A6,B6,tc);
            u=u+6;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k6)=[];
        buffreceiver=buffreceiver-k6;
        
    elseif len>2100 && len<3000
        %M=4---------------------------------------------------------------
        Mcount=Mcount+4*k4;
        pow=zeros(1,k4);
        for w=1:k4
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+4),noise(:,i-len+w),A4,B4,tc);
            u=u+4;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k4)=[];
        buffreceiver=buffreceiver-k4;
        
    else
        %M=2---------------------------------------------------------------
        Mcount=Mcount+2*k2;
        pow=zeros(1,k2);
        for w=1:k2
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+2),noise(:,i-len+w),A2,B2,tc);
            u=u+2;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k2)=[];
        buffreceiver=buffreceiver-k2;
    end
    buffhist=[buffhist length(buff)];
    power(i-len+1:i-len+w)=pow;
end

%Number of samples not sent
len=length(buff);
ns=N-(i-len);
if ns>0
    %Calculate new betaa
    betaa=maxbeta+1;
    while betaa>=maxbeta
        u=u+1;
        d=pulse*fading(u);
        betaa=pulse/d;
    end

    buff=[buff x(i+1:end)];
    pow=zeros(1,ns);
    Mcount=Mcount+16*ns;
    for w=1:ns
        [out(i-len+w) pow(w)]=sendd(buff(1),betaa,fading(u+1:u+16),noise(:,i-len+w),A16,B16,tc);
        u=u+16;
        %smallfading(i-len+w)=fading(u)*betaa;
        buff(1)=[];
    end
    power(i-len+1:i-len+w)=pow;
end

power;
Meffective=Mcount/N
power=(1/Mcount)*sum(power);
outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));

%Because the signal is a song we have to do this
% out=out/(max(abs(out))+1e-6);

% but ther is a problem, if k is not a multiple of k2length, there are
% still some samples in the buffer that need to be sent


%#1: We supouse that the faiding doesn't change betwen each sample.
%#2: We do this because the fading only has to change each 16 (Mmax)
%    retransmissions. This means that for eample, if we are sending kt
%    samples the fading will change for every sample, and if we are using
%    k4 the fading will change every 4 samples (4*M=16). We only can do
%    this for number that can divide 16


function [y,pow]=sendd(x,betaa,fading,noise,A,B,tc)

% This function will transmit one sample x using M retransmissions. This
% sample is affected by the channel fading and noise
%
% x: is the sample to be sent
% betaa: amplification factor calculated to compensate the fading
% fading: channel fading for this time instant
% noise: vector which contains gaussian noise (M+1) samples
% A and B: vectors with the coefficient for the retransmissions
global buffreceiver;

if buffreceiver<=floor(tc*16/14)
    M=16;
elseif buffreceiver>floor(tc*16/14) && buffreceiver<=300
    M=14;
elseif buffreceiver>300 && buffreceiver<=600
    M=12;
elseif buffreceiver>600 && buffreceiver<=1000
    M=10;
elseif buffreceiver>1000 && buffreceiver<=1500
    M=8;
elseif buffreceiver>1500 && buffreceiver<=2100
    M=6;
elseif buffreceiver>2100 && buffreceiver<3000
    M=4;
else
    M=2;
end

pow=zeros(1,M);
d=0;y=0;
for i=2:M+1
    a=(x-d)*betaa*A(i);
    %b=(a+noise(i,n))*fading(u);
    pow(i-1)=a;
    b=a*fading(i-1)+noise(i);
    c=b*B(i);
    y=c+y;
    d=y*fading(i-1)*betaa;
end
pow=sum(pow.^2);





