function [out,SNR,buffhist,smallfading,power]=systembufferM2(x,Es,SigmX,SigmN,noise,tc,fading,maxbeta)


% This function implements the system without noise in the feedback and
% with fading in the channel.
%
% x:input signal
% Es: Energy per source sample (0~20 dB)
% M: Expansion factor
% SigmaX: Signal variance
% SigmaN: Channel noise power
% tc (samples): Coherence time
% faiding: is a vector with the faiding components. Notice that this vector
%          has to be big enough (length(faiding)>>lengt(x))
% k1length & k2length: Are the length of the first and second part of the
%                      frame respectively
% pulse: value of the pulse sent from the receiver to the transmitter to
%        measure the fading in the channel
% flag: value of the flags used to indicate when the data is incoming. Has
% to be high but limitted by the receiver
% maxbeta: maximum value of the apmlification factor to compensate the
%          fading. Is related to the buffer size.
%
% Usage example:
% SigmaN=10^(0/10);
% noise=randn(16+1,length(x))*sqrt(SigmaN);
% [fading, tc]=rayleighfading(9,1/(44100*16),1e6);
% [out,SNR,buffhist,power]=systmwithvariablem4(x,10,0,0,noise,tc,fading,2);

global buffreceiver
buffreciever=0;

x=(x(:))';
Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
maxbeta=10^(maxbeta/10);
N=length(x);
pulse=4;
flag=3;


%---------------------------Calculate%coefficients--------------------------
M=2;
A2=zeros(1,2+1);B2=zeros(1,2+1);D4=zeros(1,2+1);
A2(1)=0;B2(1)=0;D2(1)=SigmaX;
for i=2:(2+1)
    A2(i)=sqrt((2*Esl/M)/D2(i-1));
    B2(i)=sqrt(D2(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D2(i)=D2(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end

M=4;
A4=zeros(1,4+1);B4=zeros(1,4+1);D4=zeros(1,4+1);
A4(1)=0;B4(1)=0;D4(1)=SigmaX;
for i=2:(4+1)
    A4(i)=sqrt((2*Esl/M)/D4(i-1));
    B4(i)=sqrt(D4(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D4(i)=D4(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------
% Inizialize all variables

buffhist=[];power=zeros(1,N);buff=[];
out=zeros(1,N);smallfading=zeros(1,N);
i=0;u=0;% i and u point to the last used sample in x and fading respectively
Mcount=0;% to count total M
k2=tc*2;k4=tc;
kt=k4;
buffreceiver=0;

while (N-i+length(buff))>=kt
    
    % Buffer k samples kt=(tc-1)/Mmax
    if i+kt<=N
        buff=[buff x(i+1:i+kt)];
        i=i+kt;
        buffreceiver=buffreceiver+kt;
    end
    if length(buff)<(kt)
        % If we are in this point is because (N-i+length(buff))>kt, but
        % there are les than kt-1 samples to be sent, so we have to buffer
        % untill kt
        aux=length(buff);
        buff=[buff x(i+1:i+kt-aux)];
        i=i+kt-aux;
        buffreceiver=kt;
    end
    
    %-------------------Calculate amplification factor---------------------
    u=u+1;
    d=pulse*fading(u); %Send pulse to measure fading
    betaa=pulse/d;
    while betaa>=maxbeta
        u=u+kt*4; %We advance tc, which means kt samples
        d=pulse*fading(u);
        betaa=pulse/d;
        %If betaa>maxbeta is because exists outage and wehave to resend the
        %pulse each tc
        %No we have to buffer the corresponding samples to tc --> kt
        if i+kt<=N
            buff=[buff x(i+1:i+kt)];
            i=i+kt;
            buffreceiver=buffreceiver+kt;
        end
    end
    %----------------------------------------------------------------------
    
    %Now the reciver has to know that we are sending nformation again.
    
    %-------------------------------Send flag------------------------------
    u=u+1;
    ff=flag*betaa*fading(u)+rand(1)*sqrt(SigmaN);
    if ff<2
        disp('Error in flag reception. The system does not work')
        return;
    end
    %-------------------------------------------------------------------
    len=length(buff);
    % Transmit k sample
    if len<=100
        %M=4---------------------------------------------------------------
        Mcount=Mcount+4*k4;
        pow=zeros(1,k4);
        for w=1:k4
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+4),noise(:,i-len+w),A4,B4,tc);
            u=u+4;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k4)=[];
        buffreceiver=buffreceiver-k4;
        
    else
        %M=2---------------------------------------------------------------
        Mcount=Mcount+2*k2;
        pow=zeros(1,k2);
        for w=1:k2
            [out(i-len+w) pow(w)]=sendd(buff(w),betaa,fading(u+1:u+2),noise(:,i-len+w),A2,B2,tc);
            u=u+2;
            %smallfading(i-len+w)=fading(u)*betaa;
        end
        buff(1:k2)=[];
        buffreceiver=buffreceiver-k2;
    end
    buffhist=[buffhist length(buff)];
    power(i-len+1:i-len+w)=pow;
end

%Number of samples not sent
len=length(buff);
ns=N-(i-len);
if ns>0
    %Calculate new betaa
    betaa=maxbeta+1;
    while betaa>=maxbeta
        u=u+1;
        d=pulse*fading(u);
        betaa=pulse/d;
    end

    buff=[buff x(i+1:end)];
    pow=zeros(1,ns);
    Mcount=Mcount+4*ns;
    for w=1:ns
        [out(i-len+w) pow(w)]=sendd(buff(1),betaa,fading(u+1:u+16),noise(:,i-len+w),A4,B4,tc);
        u=u+4;
        %smallfading(i-len+w)=fading(u)*betaa;
        buff(1)=[];
    end
    power(i-len+1:i-len+w)=pow;
end

power;
Meffective2=Mcount/N
power=(1/Mcount)*sum(power);
outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));

%Because the signal is a song we have to do this
% out=out/(max(abs(out))+1e-6);

% but ther is a problem, if k is not a multiple of k2length, there are
% still some samples in the buffer that need to be sent


%#1: We supouse that the faiding doesn't change betwen each sample.
%#2: We do this because the fading only has to change each 16 (Mmax)
%    retransmissions. This means that for eample, if we are sending kt
%    samples the fading will change for every sample, and if we are using
%    k4 the fading will change every 4 samples (4*M=16). We only can do
%    this for number that can divide 16


function [y,pow]=sendd(x,betaa,fading,noise,A,B,tc)

% This function will transmit one sample x using M retransmissions. This
% sample is affected by the channel fading and noise
%
% x: is the sample to be sent
% betaa: amplification factor calculated to compensate the fading
% fading: channel fading for this time instant
% noise: vector which contains gaussian noise (M+1) samples
% A and B: vectors with the coefficient for the retransmissions
global buffreceiver;

if buffreceiver<=100
    M=4;
else
    M=2;
end

pow=zeros(1,M);
d=0;y=0;
for i=2:M+1
    a=(x-d)*betaa*A(i);
    %b=(a+noise(i,n))*fading(u);
    pow(i-1)=a;
    b=a*fading(i-1)+noise(i);
    c=b*B(i);
    y=c+y;
    d=y*fading(i-1)*betaa;
end
pow=sum(pow.^2);





