function [out,SNR,buffhist,smallfading,power]=systembufferframe(x,Es,M,SigmX,SigmN,noise,tc,fading,maxbeta)

% This function implements the system without noise in the feedback and
% with fading in the channel.
%
% x:input signal
% Es: Energy per source sample (0~20 dB)
% M: Expansion factor
% SigmaX: Signal variance
% SigmaN: Channel noise power
% tc (samples): Coherence time
% faiding: is a vector with the faiding components. Notice that this vector
%          has to be big enough (length(faiding)>>lengt(x))
% k1 & k2: Are the length of the first and second part of the
%                      frame respectively
% pulse: value of the pulse sent from the receiver to the transmitter to
%        measure the fading in the channel
% flag: value of the flags used to indicate when the data is incoming. Has
% to be high but limitted by the receiver
% maxbeta: maximum value of the apmlification factor to compensate the
%          fading. Is related to the buffer size.
%
% Usage example:
% SigmaN=10^(0/10);
% noise=randn(16+1,length(x))*sqrt(SigmaN);
% [fading, tc]=rayleighfading(19.2,1/(44100*16),1e6);
% [out,SNR,buffhist,power]=idealsystemfading(x,10,4,0,0,noise,tc,fading,2);

x=(x(:))';
Esl=10^(Es/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
maxbeta=10^(maxbeta/10);
N=length(x);
pulse=4;
flag=3;
k1=fix(tc/2);k2=k1;

%---------------------------Calculate%coefficients--------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=0;B(1)=0;D(1)=SigmaX;

for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------
buffhist=[];power=zeros(1,N);
out=zeros(1,N);smallfading=zeros(1,N);
i=0;u=0;% i and u point to the last used sample in x and fading respectively
buff=[];
%k1s=fix(k1/M);k2s=fix(k2/M);
k1s=k1;k2s=k2;

while (N-i+length(buff))>=k1s
     
    % Buffer k1/M samples
    if i+k1s<=N
        buff=[buff x(i+1:i+k1s)];
        i=i+k1s;
    end
    if length(buff)<(k1s)
        % If we are in this point is because (N-i+length(buff))>k1, but
        % there are les than k1 samples to be sent, so we have to buffer
        % untill k
        aux=length(buff);
        buff=[buff x(i+1:i+k1s-aux)];
        i=i+k1s-aux;
    end
    
    %-------------------Calculate amplification factor---------------------
    u=u+1;
    d=pulse*fading(u); %Send pulse to measure fading
    betaa=pulse/d;
    while betaa>=maxbeta
        u=u+tc*M;  %advance tc samples == tc*M intervals
        d=pulse*fading(u);
        betaa=pulse/d;
        %If betaa>maxbeta is because exists outage and wehave to resend the
        %pulse each tc
        %No we have to buffer the corresponding samples to tc --> kt
        if i+k1s<=N
            buff=[buff x(i+1:i+k1s)];
            i=i+k1s;
        end
    end
    %----------------------------------------------------------------------
    
    %Now the reciver has to know that we are sending nformation again.
    
    %-------------------------------Send flag------------------------------
    u=u+1;
    ff=flag*betaa*fading(u)+rand(1)*sqrt(SigmaN);
    if ff<2
        disp('Error in flag reception. The system does not work')
        return;
    end
    %-------------------------------------------------------------------
    
    len=length(buff);
    for w=1:k1s
        [out(i-len+w) pow]=sendd(buff(w),M,betaa,fading(u+1:u+M),noise(:,i-len+w),A,B);
        u=u+M;
        %smallfading(i-len+w)=fading(u)*betaa;
        power(i-len+w)=pow;
    end
    buff(1:k1s)=[];
        
    %If there is still something in the buffer we have to send it
    len=length(buff);
    buffhist=[buffhist len];
    if len>=k2s
        %send(falg)-------------------
        for w=1:k2s
            [out(i-len+w) pow]=sendd(buff(w),M,betaa,fading(u+1:u+M),noise(:,i-len+w),A,B);
            u=u+M;
            %smallfading(i-len+w)=fading(u)*betaa;
            power(i-len+w)=pow;
        end
        buff(1:k2s)=[];
    else
        u=u+k2s*M;
    end
end

len=length(buff);
ns=N-(i-len);
if ns>0
    %Calculate new betaa
    betaa=maxbeta+1;
    while betaa>=maxbeta
        u=u+1;
        d=pulse*fading(u);
        betaa=pulse/d;
    end
    buff=[buff x(i+1:end)];
    
    for w=1:ns
        [out(i-len+w) pow]=sendd(buff(1),M,betaa,fading(u+1:u+M),noise(:,i-len+w),A,B);
        u=u+M;
        %smallfading(i-len+w)=fading(u)*betaa;
        buff(1)=[];
        power(i-len+w)=pow;
    end
end

power=(1/(N*M))*sum(power);
outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));

%Because the signal is a song we have to do this
%out=out/(max(abs(out))+1e-3);

% but ther is a problem, if k is not a multiple of k2length, there are
% still some samples in the buffer that need to be sent


function [y,pow]=sendd(x,M,betaa,fading,noise,A,B)

% This function will transmit one sample x using M retransmissions. This
% sample is affected by the channel fading and noise
%
% x: is the sample to be sent
% betaa: amplification factor calculated to compensate the fading
% fading: channel fading for this time instant
% noise: vector which contains gaussian noise (M+1) samples
% A and B: vectors with the coefficient for the retransmissions
u=0;
p=zeros(1,M);
d=0;y=0;
for i=2:M+1
    u=u+1;
    a=(x-d)*betaa*A(i);
    %b=(a+noise(i,n))*fading(u);
    p(i-1)=a;
    c=(a*fading(u)+noise(i))*B(i);
    y=c+y;
    d=y*fading(u)*betaa;
end

pow=sum(p.^2);