function [out,SNR]=systemfading(x,Es,M,SigmX,SigmN,noise,fading,tc,betamax)

% Function to send signal x through the system without buffering
%
% Es: Energy per source sample (0~20 dB)
% M: Expansion factor
% N: Number of samles
% SigmaX: Signal variance
% SigmaN: Channel noise power
% noise=randn(M+1,N)*sqrt(SigmaN);
% fading: fading samples generated from genfading.m
% tc: Coherence time (samples)
% betamax(dB): Outage threshold
%
% Example:
% noise=randn(16+1,length(x))*sqrt(SigmaN);
% [fading, tc]=genfading('rice',19,1/44100,1e5,10);
% [out,SNR]=systemfading(x,10,16,0,0,noise,fading,1,6);

x=(x(:))';
Esl=10^(Es/10);
betamaxl=10^(betamax/10);
SigmaX=10^(SigmX/10);
SigmaN=10^(SigmN/10);
N=length(x);

%--------------------------Outage probability-------------------------------
count=0;
for i=1:length(fading)
    if fading(i)<(1/betamaxl)
        count=count+1;
    end
end
poutage=count/length(fading);
disp('The outage probability  P_out(betamax) = ');
disp(poutage);


%------------------------Calculate Coeficients------------------------------
A=zeros(1,M+1);B=zeros(1,M+1);D=zeros(1,M+1);
A(1)=0;B(1)=0;D(1)=SigmaX;

for i=2:(M+1)
    A(i)=sqrt((2*Esl/M)/D(i-1));
    B(i)=sqrt(D(i-1)*(2*Esl/M))/((2*Esl/M)+SigmaN);
    D(i)=D(i-1)*SigmaN/((2*Esl/M)+SigmaN);
end
%--------------------------------------------------------------------------

out=zeros(1,N);
fadingcount=1;
pulse=3;
resfading=zeros(1,N);


%----------------------------------->Start<--------------------------------
d=pulse*fading(1); %Send pulse to measure fading
betaa=pulse/d;
if betaa>betamax
    betaa=betamax;
end

for n=1:N
    d=0;
    y=0;
    if fadingcount==tc
        fadingcount=0;
        d=pulse*fading(n); %Send pulse to measure fading
        betaa=pulse/d;
        if betaa>betamax
            betaa=betamax;
        end
    end
    
    resfading(n)=betaa;

    for i=2:M+1
        d=y*fading(n)*betaa;
        a=(x(n)-d)*A(i);
        b=a*betaa*fading(n)+noise(i,n);
        c=b*B(i);
        y=c+y;
    end
    out(n)=y;
    fadingcount=fadingcount+1;
end

plot(fading)
hold on;
plot(1./resfading,'r')
hold off,

outputnoise=x-out;
SNR=10*log10(SigmaX/var(outputnoise));